require 'spec_helper'

require 'tablets/data/query'

RSpec.describe Tablets::Data::Query do
  subject { described_class.new(relation, params, columns) }

  let(:relation) { 'relation' }
  let(:params) { {} }
  let(:columns) { :columns }

  before do
    allow(Tablets::Data::Processing::Order).to receive(:apply)
      .with(params, columns, relation).and_return(relation)
    allow(Tablets::Data::Processing::Filter).to receive(:apply)
      .with(params, columns, relation).and_return(relation)
    allow(Tablets::Data::Processing::Paginate).to receive(:apply)
      .with(params, columns, relation).and_return(relation)
  end

  describe '#fetch' do
    let!(:result) { subject.fetch }

    let(:params) { { order: :order, search: :search } }

    it 'returns relation' do
      expect(result).to eq(relation)
    end

    it 'applies order' do
      expect(Tablets::Data::Processing::Order).to have_received(:apply)
        .with(params, columns, relation)
    end

    it 'applies filter' do
      expect(Tablets::Data::Processing::Filter).to have_received(:apply)
        .with(params, columns, relation)
    end

    it 'applies paginate' do
      expect(Tablets::Data::Processing::Paginate).to have_received(:apply)
        .with(params, columns, relation)
    end

    context 'without order' do
      let(:params) { { search: :search } }

      it 'not applies order' do
        expect(Tablets::Data::Processing::Order).to_not have_received(:apply)
      end
    end

    context 'without search' do
      let(:params) { { order: :order } }

      it 'not applies order' do
        expect(Tablets::Data::Processing::Filter).to_not have_received(:apply)
      end
    end

    context 'with negative length' do
      let(:params) { { length: '-1' } }

      it 'not applies order' do
        expect(Tablets::Data::Processing::Paginate).to_not have_received(:apply)
      end
    end
  end

  describe '#total' do
    let(:total) { :total }

    before do
      allow(relation).to receive(:count).with(:all).and_return(total)
    end

    let!(:result) { subject.total }

    it 'returns total' do
      expect(result).to eq(total)
    end
  end

  describe '#filtered' do
    let(:params) { { search: :search } }

    let(:total) { :total }

    before do
      allow(relation).to receive(:count).with(:all).and_return(total)
    end

    let!(:result) { subject.filtered }

    it 'applies filter' do
      expect(Tablets::Data::Processing::Filter).to have_received(:apply)
        .with(params, columns, relation)
    end

    context 'without search' do
      let(:params) { {} }

      it 'not applies order' do
        expect(Tablets::Data::Processing::Filter).to_not have_received(:apply)
      end
    end

    it 'returns total' do
      expect(result).to eq(total)
    end
  end
end
