require 'spec_helper'

require 'tablets/utils/config'

RSpec.describe Tablets::Utils::Config do
  subject { described_class.new(&block) }

  def hash
    subject.instance_eval { @hash }
  end

  describe '#initialize' do
    context 'with block' do
      let(:block) { proc { var('val') } }

      it 'applies block to config' do
        expect(hash).to_not be_empty
      end
    end

    context 'without block' do
      let(:block) { nil }

      it 'initializes empty config' do
        expect(hash).to be_empty
      end
    end
  end

  describe '#apply' do
    let(:block) { nil }
    let(:another_block) { proc { var('val') } }

    it 'applies block' do
      subject.apply(&another_block)

      expect(hash).to_not be_empty
    end
  end

  describe '#has?' do
    context 'if value exists' do
      let(:block) { proc { var('val') } }

      it 'returns true' do
        expect(subject.has?(:var)).to eq(true)
      end
    end

    context 'if value not exists' do
      let(:block) { nil }

      it 'returns false' do
        expect(subject.has?(:var)).to eq(false)
      end
    end
  end

  describe '#get' do
    context 'if value exists' do
      let(:block) { proc { var('val') } }

      it 'returns value' do
        expect(subject.get(:var)).to eq('val')
      end
    end

    context 'if value not exists' do
      let(:block) {}

      it 'raises error' do
        expect do
          subject.get(:var)
        end.to raise_error(ArgumentError)
      end
    end

    context 'if value not exists but have default' do
      let(:block) {}

      it 'returns default' do
        expect(subject.get(:var, 'default')).to eq('default')
      end
    end

    context 'if value not exists but have default block' do
      let(:block) {}

      it 'returns default block return value' do
        expect(subject.get(:var) { 'default' }).to eq('default')
      end
    end
  end

  describe '#call' do
    context 'if callback exists' do
      let(:block) { proc { var { 'val' } } }

      it 'returns callback return value' do
        expect(subject.call(:var)).to eq('val')
      end
    end

    context 'if callback not exists' do
      let(:block) {}

      it 'raises error' do
        expect do
          subject.call(:var)
        end.to raise_error(ArgumentError)
      end
    end

    context 'if value not exists but have default block' do
      let(:block) {}

      it 'returns default block return value' do
        expect(subject.call(:var) { 'default' }).to eq('default')
      end
    end
  end
end
